<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Administrador extends CI_Controller {
	private $tipoadmin = 1;
	
	public function __construct(){
		parent::__construct();			
		/* Comprovació d'acces en el constructor per a totes les funcions */
		if(!is_logged() || $this->session->userdata('tipus')!=$this->tipoadmin){
			redirect(base_url());
			exit;
		}
	}
	
	/* 
	 * Metode per defecte mostra dashboard
	 */
	public function index()
	{
		$data['page']="administrador/dashboard";
		$this->load->view('home', $data);		
	}
	
	/* 
	 * Llistat d'alumnes
	 */
	public function alumnes()
	{
		$this->load->model('Usuari_model','',TRUE);
		$lista['alumnes'] = $this->Usuari_model->llistarUsuaris(3);
		$data['llistat_alumnes'] = $this->load->view('administrador/alumnes/llistat_alumnes', $lista, TRUE);
		 
		$this->load->model('Assignatura_model','',TRUE);
		$data['assignatures'] = $this->Assignatura_model->llistarAssignatures();
		
		$data['page']="administrador/alumnes/admin_alumnes";
		$this->load->view('home', $data);	
	}
	
	/* 
	 * Metode AJAX que retorna un json amb el llistat d'alumnes si no hi ha error en la creacio del alumne
	 * S'utilitza per crear i editar alumnes, rep les dades per post i les guarda
	 */
	public function alumne_action()
	{
		$this->load->model('Usuari_model','',TRUE);		
		$sqldata['nif'] = filter_var($this->input->post('nif'), FILTER_SANITIZE_STRING);
		$sqldata['nom'] = filter_var($this->input->post('nom'), FILTER_SANITIZE_STRING);
		$sqldata['cognoms'] = filter_var($this->input->post('cognoms'), FILTER_SANITIZE_STRING);
		$sqldata['mail'] = filter_var($this->input->post('mail'), FILTER_SANITIZE_EMAIL);
		$assignatura = $this->input->post('assignatura');
		$action = $this->input->post('action');
			
		/* fallo de recepcio de dades*/	
		$errmsg = "";	
		foreach($sqldata as $key => $val){
			if(empty($val)){
				$errmsg .= "Error al rebre el camp $key \n\r";
			}
		}
		
		if(empty($assignatura) || count($assignatura)==0){
			$errmsg .= "No s'ha rebut cap assignatura";
		}
			
		if($errmsg != ""){
			$this->_print_error($errmsg);
		}
		
		if($action == "add"){
			$sqldata['tipus'] = 3;
			$clau = cadena_aleatoria(8);
			//$clau = "123123" descomentar per probar sense servidor de correu
			$sqldata['clau'] = sha1($clau);
			/* comprobar email ja existent */
			if($this->Usuari_model->existeixMailUsuari($sqldata['mail'])){
				$this->_print_error("Ja existeix un usuari amb aquest mail");
			}
			/* comprobar nif ja existent */
			if($this->Usuari_model->existeixNifUsuari($sqldata['nif'])){
				$this->_print_error("Ja existeix un usuari amb aquest nif");
			}
			/* Si falla al introduir */			
			if(!$idalumne = $this->Usuari_model->crearUsuari($sqldata)){
				$this->_print_error("Error al insertar l'Alumne");
			}	
			
			/*enviar mail amb contrasenya */
			$this->_send_pass_mail($clau,$sqldata['mail'],$sqldata['nom']);
			
		}elseif($action == "edit"){
			//falla recepcio de idusuari
			$idalumne = filter_var($this->input->post('idalumne'), FILTER_SANITIZE_NUMBER_INT);		
			if(empty($idalumne)){			
				$this->_print_error("Error al editar alumne");
			}
			
			/* Si falla al modificar */
			if(!$this->Usuari_model->editarUsuari($idalumne, $sqldata)){
				$this->_print_error("Error al editar alumne");
			}	
			$this->Usuari_model->resetUsuariAssignatures($idalumne);
		}else{
			$this->_print_error("Error inesperat");
		}
		
		/* assignaturas que cursa */
		foreach($assignatura as $ida){
			$this->Usuari_model->insertarUsuariAssignatura($idalumne, $ida);
		}
				
		/* Si arribem aqui sense errors retornem llistat de alumnes */
		$lista['alumnes'] = $this->Usuari_model->llistarUsuaris(3);
		$alumnes = $this->load->view('administrador/alumnes/llistat_alumnes', $lista, TRUE);	
		$json = array('error'=>'0', 'alumnes'=>$alumnes);
		echo json_encode($json); 
	}
	
	/* 
	 * Metode AJAX que retorna un json amb la informació del alumne
	 * Per editar les dades d'un alumne en ajax
	 */
	public function alumne_info(){
		$this->load->model('Usuari_model','',TRUE);
		
		$idalumne = filter_var($this->input->post('idalumne'), FILTER_SANITIZE_NUMBER_INT);		
		if(empty($idalumne)){			
			$this->_print_error("Error al recuperar la informació de l'alumne");
		}
		//si existeix retornem sino error
		if($alumne = $this->Usuari_model->dadesUsuari($idalumne)){
			if($alumne['assignatures'] = $this->Usuari_model->assignaturesUsuari($idalumne))
			$json = array('error'=>'0', 'alumne'=>$alumne);
			echo json_encode($json);
		}else{
			$this->_print_error("No s'ha trobat l'alumne");
		}
	}
	
	/* 
	 * Metode AJAX que retorna un json confirman la eliminació junt amb el nou llistat del alumne o error
	 * Per eliminar alumnes per ajax
	 */
	public function alumne_delete(){
		$this->load->model('Usuari_model','',TRUE);
				
		$idalumne = filter_var($this->input->post('idalumne'), FILTER_SANITIZE_NUMBER_INT);		
		if(empty($idalumne)){				
			$this->_print_error("Error al eliminar l'alumne");
		}
		if($this->Usuari_model->deleteAlumne($idalumne)===FALSE){
			$this->_print_error("Error al eliminar l'alumne");
		}
		
		$lista['alumnes'] = $this->Usuari_model->llistarUsuaris(3);
		$alumnes = $this->load->view('administrador/alumnes/llistat_alumnes', $lista, TRUE);	
		$json = array('error'=>'0', 'alumnes'=>$alumnes);
		echo json_encode($json); 	
	}
	
	/* 
	 * Llistat de professors
	 */
	public function professors()
	{
		$this->load->model('Usuari_model','',TRUE);
		$lista['professors'] = $this->Usuari_model->llistarUsuaris(2);
		$data['llistat_professors'] = $this->load->view('administrador/professors/llistat_professors', $lista, TRUE);
		
		$this->load->model('Assignatura_model','',TRUE);
		$data['assignatures'] = $this->Assignatura_model->llistarAssignatures();
				
		$data['page']="administrador/professors/admin_professors";
		$this->load->view('home', $data);	
	}
	
	/* 
	 * Metode AJAX que retorna un json amb el llistat de professors si no hi ha error en la creacio del professor
	 * S'utilitza per crear i editar professors, rep les dades per post i les guarda
	 */
	public function professor_action()
	{
		$this->load->model('Usuari_model','',TRUE);		
		$sqldata['nif'] = filter_var($this->input->post('nif'), FILTER_SANITIZE_STRING);
		$sqldata['nom'] = filter_var($this->input->post('nom'), FILTER_SANITIZE_STRING);
		$sqldata['cognoms'] = filter_var($this->input->post('cognoms'), FILTER_SANITIZE_STRING);
		$sqldata['mail'] = filter_var($this->input->post('mail'), FILTER_SANITIZE_EMAIL);
		$assignatura = $this->input->post('assignatura');
		$action = $this->input->post('action');
			
		/* fallo de recepcio de dades*/	
		$errmsg = "";	
		foreach($sqldata as $key => $val){
			if(empty($val)){
				$errmsg .= "Error al rebre el camp $key \n\r";
			}
		}
		
		if(empty($assignatura) || count($assignatura)==0){
			$errmsg .= "No s'ha rebut cap assignatura";
		}
			
		if($errmsg != ""){
			$this->_print_error($errmsg);
		}
		
		if($action == "add"){
			$sqldata['tipus'] = 2;
			$clau = cadena_aleatoria(8);
			//$clau = "123123" descomentar per probar sense servidor de correu
			$sqldata['clau'] = sha1($clau);
			/* comprobar email ja existent */
			if($this->Usuari_model->existeixMailUsuari($sqldata['mail'])){
				$this->_print_error("Ja existeix un usuari amb aquest mail");
			}
			/* comprobar nif ja existent */
			if($this->Usuari_model->existeixNifUsuari($sqldata['nif'])){
				$this->_print_error("Ja existeix un usuari amb aquest nif");
			}
			/* Si falla al introduir */			
			if(!$idprofessor = $this->Usuari_model->crearUsuari($sqldata)){
				$this->_print_error("Error al insertar el professor");
			}	
			
			/*enviar mail amb contrasenya */
			$this->_send_pass_mail($clau,$sqldata['mail'],$sqldata['nom']);
			
		}elseif($action == "edit"){
			//falla recepcio de idusuari
			$idprofessor = filter_var($this->input->post('idprofessor'), FILTER_SANITIZE_NUMBER_INT);		
			if(empty($idprofessor)){			
				$this->_print_error("Error al editar professor");
			}
			
			/* Si falla al modificar */
			if(!$this->Usuari_model->editarUsuari($idprofessor, $sqldata)){
				$this->_print_error("Error al editar professor");
			}	
			$this->Usuari_model->resetUsuariAssignatures($idprofessor);
		}else{
			$this->_print_error("Error inesperat");
		}
		
		/* assignaturas que cursa */
		foreach($assignatura as $ida){
			$this->Usuari_model->insertarUsuariAssignatura($idprofessor, $ida);
		}
		
		/* Si arribem aqui sense errors retornem llistat de alumnes */
		$lista['professors'] = $this->Usuari_model->llistarUsuaris(2);
		$professors = $this->load->view('administrador/professors/llistat_professors', $lista, TRUE);	
		$json = array('error'=>'0', 'professors'=>$professors);
		echo json_encode($json); 
	}
	
	/* 
	 * Metode AJAX que retorna un json amb la informació del alumne
	 * Per carregar la informacio a editar del professor en ajax
	 */
	public function professor_info(){
		$this->load->model('Usuari_model','',TRUE);
		
		$idprofessor = filter_var($this->input->post('idprofessor'), FILTER_SANITIZE_NUMBER_INT);		
		if(empty($idprofessor)){			
			$this->_print_error("Error al recuperar la informació del professor");
		}
		if($professor = $this->Usuari_model->dadesUsuari($idprofessor)){
			if($professor['assignatures'] = $this->Usuari_model->assignaturesUsuari($idprofessor))
			$json = array('error'=>'0', 'professor'=>$professor);
			echo json_encode($json);
		}else{
			$this->_print_error("No s'ha trobat el professor");
		}
	}
	
	/* 
	 * Metode AJAX que retorna un json confirman la eliminació junt amb el nou llistat de professors o error en la elimincació
	 * Per eliminar professors per ajax
	 */
	public function professor_delete(){
		$this->load->model('Usuari_model','',TRUE);
				
		$idprofessor = filter_var($this->input->post('idprofessor'), FILTER_SANITIZE_NUMBER_INT);		
		if(empty($idprofessor)){				
			$this->_print_error("Error al eliminar el professor");
		}
		
		if($this->Usuari_model->deleteProfessor($idprofessor)===FALSE){
			$this->_print_error("Error al eliminar el professor");
		}
		
		$lista['professors'] = $this->Usuari_model->llistarUsuaris(2);
		$professors = $this->load->view('administrador/professors/llistat_professors', $lista, TRUE);	
		$json = array('error'=>'0', 'professors'=>$professors);
		echo json_encode($json); 	
	}
	
	/* 
	 * Llistat d'assignatures
	 */
	public function assignatures()
	{
		$this->load->model('Assignatura_model','',TRUE);		
		$lista['assignatures'] = $this->Assignatura_model->llistarAssignatures();
		$data['llistat_assignatures'] = $this->load->view('administrador/assignatures/llistat_assignatures', $lista, TRUE);
		$data['page']="administrador/assignatures/admin_assignatures";
		$this->load->view('home', $data);	
	}
	
	/* 
	 * Metode AJAX que retorna un json amb el llistat d'assignatures si no hi ha error en la creacio de l'assignatura
	 * S'utilitza per crear i editar assignatures, rep les dades per post i les guarda
	 */
	public function assignatura_action()
	{
		$this->load->model('Assignatura_model','',TRUE);		
		$nom = filter_var($this->input->post('nom'), FILTER_SANITIZE_STRING);
		$action = $this->input->post('action');
		if(empty($nom)){
			$this->_print_error("Error al rebre el camp nom");
		}
		
		$sqldata = array('nom' => $nom);
		if($action == "add"){
			/* Si falla al introduir */			
			if(!$this->Assignatura_model->crearAssignatura($sqldata)){
				$this->_print_error("Error al insertar la assignatura");
			}			
		}elseif($action == "edit"){
			//falla idassignatura
			$idassignatura = filter_var($this->input->post('idassignatura'), FILTER_SANITIZE_NUMBER_INT);		
			if(empty($idassignatura)){			
				$this->_print_error("Error al editar assignatura");
			}
			
			/* Si falla al modificar */
			if(!$this->Assignatura_model->editarAssignatura($idassignatura, $sqldata)){
				$this->_print_error("Error al editar assignatura");
			}		
		}else{
			$this->_print_error("Error inesperat");
		}
		
		/* Si arribem aqui sense errors retornem llistat de assignatures */
		$lista['assignatures'] = $this->Assignatura_model->llistarAssignatures();
		$assignatures = $this->load->view('administrador/assignatures/llistat_assignatures', $lista, TRUE);	
		$json = array('error'=>'0', 'assignatures'=>$assignatures);
		echo json_encode($json);
	}
	
	/* 
	 * Metode AJAX que retorna un json amb la informació de l'assignatura
	 * Per carregar la informacio a editar de la assignatura en ajax
	 */
	public function assignatura_info(){
		$this->load->model('Assignatura_model','',TRUE);
		
		$idassignatura = filter_var($this->input->post('idassignatura'), FILTER_SANITIZE_NUMBER_INT);		
		if(empty($idassignatura)){			
			$this->_print_error("Error al recuperar la informació de la assignatura");
		}
		if($assignatura = $this->Assignatura_model->dadesAssignatura($idassignatura)){
			$json = array('error'=>'0', 'assignatura'=>$assignatura);
			echo json_encode($json);
		}else{
			$this->_print_error("No s'ha trobat l'assignatura");
		}
	}
	
	/* 
	 * Metode AJAX que retorna un json confirman la eliminació junt amb el nou llistat d'assignatures o error en la elimincació
	 * Per eliminar assignatures per ajax
	 */
	public function assignatura_delete(){
		$this->load->model('Assignatura_model','',TRUE);
				
		$idassignatura = filter_var($this->input->post('idassignatura'), FILTER_SANITIZE_NUMBER_INT);		
		if(empty($idassignatura)){			
			$this->_print_error("Error al eliminar la assignatura");
		}
		if($this->Assignatura_model->deleteAssignatura($idassignatura)===FALSE){
			$this->_print_error("Error al eliminar l'assignatura");
		}
		
		$lista['assignatures'] = $this->Assignatura_model->llistarAssignatures();
		$assignatures = $this->load->view('administrador/assignatures/llistat_assignatures', $lista, TRUE);	
		$json = array('error'=>'0', 'assignatures'=>$assignatures);
		echo json_encode($json);	
	}	
	
	/* 
	 * Metode per enviar mail personalitzat als usuaris en la seva creació adjuntant la contrasenya 
	 */
	private function _send_pass_mail($pass,$mail,$name){
		$mail_msg = "Hola $name, \n " .
					"Has estat donat d'alta com a usuari de GenQuiz' \n\n " .
					"Les teves dades d'acces son \n\n" .
					"Usuari: $mail \n Contrasenya: $pass \n\n " .
					"Et recomenem que canviis la contrasenya la primera veguada que entris. \n " .
					"Gracies";
        $this->load->library('email');		
        $this->email->from('no-reply@tfc.enredats.com', 'GenQuiz');
        $this->email->to($mail);
        $this->email->subject("Contrasenya provisional genquiz");
        $this->email->message($mail_msg);
        if ( ! $this->email->send())
		{
    		return false;
		}
		
		return true;
	}
	
	/* 
	 * Retorna errors en format json per ser interpretats per el js de les peticions ajax 
	 */
	private function _print_error($msg=""){
		$tmpl_msg = $this->load->view('modals/errors', array("error_msg"=>$msg), TRUE);
        $json = array('error'=>'1', 'msg'=>$tmpl_msg);
        echo json_encode($json);
        exit;
    }		
	
}

?>
