<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Alumne extends CI_Controller {
	private $tipo = 3;
	
	public function __construct(){
		parent::__construct();			
		/* Comprovació d'acces */
		if(!is_logged() || $this->session->userdata('tipus')!=$this->tipo){
			redirect(base_url());
			exit;
		}
	}
	/* alumnes dashboard */
	public function index()
	{
		$data['page']="alumne/dashboard";
		$this->load->view('home', $data);		
	}
	
	/* 
	 * Llistat d'examens actius
	 */
	public function examens_actius()
	{
		$this->load->model('Examen_model','',TRUE);
		$data['examens'] = $this->Examen_model->examensActiusAlumne($this->session->userdata('idusuario'));
		$data['page']="alumne/examens_actius";
		$this->load->view('home', $data);		
	}
		
	
	/* 
	 * Llistat d'examens acabats
	 */
	public function examens_acabats()
	{
		$this->load->model('Examen_model','',TRUE);
		$data['examens'] = $this->Examen_model->examensAcabatsAlumne($this->session->userdata('idusuario'));
		$data['page']="alumne/examens_acabats";
		$this->load->view('home', $data);		
	}
	
	/* 
	 * Mostra la plana d'examen per a realitzar-lo fent les comprobacions oportunes avans
	 */
	public function realitzar_examen($idexamen = null){
		//Si no hi ha id de examen error
		if($idexamen == null){
			$this->_show_html_error("Error: Examen no especificat:","No s'ha rebut l'examen que vols realitzar");
			return false;	
		}
		//Comprobem si es valid!
		$this->load->model('Examen_model','',TRUE);
		$valid = $this->Examen_model->examenValid($idexamen, $this->session->userdata('idusuario'));
		if($valid==null || $valid['fet']!=null){
			$this->_show_html_error("Error: No tens access per realitzar aquest examen:","Es posible que el examen no existeixi, hagi caducat o no hagi estat obert, l'hagis ja realitzat o no et pertoqui");
			return false;	
		}
		//Si es un examen valid, no l'ha realitzat ja i es de una assignatura de la que esta matriculat:		
		$data['examen'] = $this->Examen_model->infoExamen($idexamen);	
		$data['preguntes'] = $this->Examen_model->llistatPreguntes($idexamen);
		foreach($data['preguntes'] as $key => $p){
			$data['preguntes'][$key]['respostes'] = $this->Examen_model->llistatRespostes($p['idpregunta']);
		}
		$data['page']="alumne/realitzar_examen";
		$this->load->view('home', $data);			
	}
	
	/* 
	 * Rep les respostes d'un examen per ajax i les guarda en la base de dades fent les comprovacions avans
	 */
	public function realitzar_examen_submit($idexamen = null){
		$this->load->model('Examen_model','',TRUE);
		//Si no hi ha id de examen error
		if($idexamen == null){
			$this->_print_error("Error id examen");
		}
		//Comprobem si es valid!
		$valid = $this->Examen_model->examenValid($idexamen, $this->session->userdata('idusuario'));
		if($valid==null || $valid['fet']!=null){
			$this->_print_error("Error examen ja realitzat o no valid pel usuari");
		}
		$respostes = ($this->input->post());
		foreach($respostes as $key => $idresposta){
			$tmp = explode("_",$key);
			if(isset($tmp[0],$tmp[1]) && $tmp[0]=="p" && is_numeric($tmp[1])){
				$idpregunta = $tmp[1];
				$sql[] = '("'.$this->session->userdata('idusuario').'", "'.$idexamen.'", "'.$idpregunta.'", "'.$idresposta.'")';
			}
		}
		if(!$this->Examen_model->insertarRespostesAlumne($sql, $this->session->userdata('idusuario'), $idexamen)){
			$this->_print_error("Error al guardar les respostes, contacte amb el servei tecnic");
		}
		
		/*respostes rebudes correctament*/
		$ok_modal = $this->load->view('modals/respostes_rebudes','',TRUE);
		$json = array('error'=>'0', 'msg'=>$ok_modal);
        echo json_encode($json);        
    }	
    
    /* 
	 * Mostrar el examen corregit. Carregar preguntes i respostes del examen i comparar amb les del alumne (en la vista)
	 */
    public function resultat_examen($idexamen = null){    	
    	$this->load->helper('correccio');
		$this->load->model('Examen_model','',TRUE);
		$this->load->model('Usuari_model','',TRUE);
		$idexamen = filter_var($idexamen, FILTER_SANITIZE_NUMBER_INT);
		$idalumne = $this->session->userdata('idusuario');
		
		if($idexamen==null || ($infoexamen = $this->Examen_model->infoExamenAlumne($idexamen,$idalumne))==null){
			$this->_show_html_error("Error examen no valid", "No ha acabat el termini de realització o no tens acces a aquest examen");
			return false;
		}
		$data['examen']=$infoexamen;
		
		if(($infoalumne = $this->Examen_model->alumneNotaExamen($idexamen,$idalumne))==null){
			$this->_show_html_error("Error alumne no valid", "El alumne no es valid");
			return false;
		}
		$data['alumne'] = $infoalumne;
		
		$data['preguntes'] = $this->Examen_model->llistatPreguntes($idexamen);
		foreach($data['preguntes'] as $key => $p){
			$data['preguntes'][$key]['respostes'] = $this->Examen_model->llistatRespostes($p['idpregunta']);
		}	
		
		$alumne_respostes = $this->Examen_model->respostesAlumneExamen($idexamen, $idalumne);
		$data['a_resp'] = adaptar_array_respostes($alumne_respostes);
	
		$data['page']="alumne/respostes_alumne";
		$this->load->view('home', $data);
    }
    
    /* 
	 * Retorna errors en format json per ser interpretats per el js de les peticions ajax 
	 */
    private function _print_error($msg=""){
		$tmpl_msg = $this->load->view('modals/errors', array("error_msg"=>$msg), TRUE);
        $json = array('error'=>'1', 'msg'=>$tmpl_msg);
        echo json_encode($json);
        exit;
    }
    
    /* 
	 * Retorna errors en format html codificat en json per als colorbox. 
	 */
    private function _show_html_error($titul="", $msg=""){
		$this->load->view('home', array("page"=>"alumne/error","titul"=>$titul,"msg"=>$msg));
    }	
}

?>
