<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Missatgeria extends CI_Controller {
	private $tipus;
	
	public function __construct(){
		parent::__construct();			
		/* Comprovació d'acces (nomes alumnes i professors) */
		if(!is_logged() || ($this->session->userdata('tipus')!=2 && $this->session->userdata('tipus')!=3)){
			redirect(base_url());
			exit;
		}
		$tipususuari = $this->config->item('tipus_usuari');
		$this->tipus = $tipususuari[$this->session->userdata('tipus')];
	}
	
	/* 
	 * Per defecte mostrem la bandeja de missatges rebuts 
	 */
	public function index()
	{
		$this->load->model('Missatge_model','',TRUE);		
		$data['missatges'] = $this->Missatge_model->missatgesRebuts($this->session->userdata('idusuario'));
		$data['page']=$this->tipus."/missatgeria/rebuts";
		$this->load->view('home', $data);		
	}
	
	/* 
	 * Metode de llistat de missatges enviats
	 */
	public function enviats()
	{
		$this->load->model('Missatge_model','',TRUE);		
		$data['missatges'] = $this->Missatge_model->missatgesEnviats($this->session->userdata('idusuario'));
		$data['page']=$this->tipus."/missatgeria/enviats";
		$this->load->view('home', $data);		
	}
	
	/* 
	 * Ens retorna la modal amb les dades omplertes per fer el enviament de missatges, tot codificat en JSON 
	 */
	public function enviar_missatge($iddestinatari = null){
		$iddestinatari = filter_var($iddestinatari, FILTER_SANITIZE_NUMBER_INT);		
		if(empty($iddestinatari)){
			$this->load->view('modals/enviar_missatge_error');
			return false;
		}
		
		//Si no existeix el destinatari error
		$this->load->model('Usuari_model','',TRUE);
		$destinatariInfo = $this->Usuari_model->dadesUsuari($iddestinatari);
		if($destinatariInfo == null){
			$this->load->view('modals/enviar_missatge_error');
			return false;
		}	
		
		//Dades per omplir la vista		
		$remitentnom = $this->session->userdata('nom'). " " . $this->session->userdata('cognoms');
		$destinatarinom = $destinatariInfo['nom']. " " .$destinatariInfo['cognoms'];		
		
		$data = array('iddestinatari'=>$iddestinatari,'destinatari'=>$destinatarinom,'remitent'=>$remitentnom);
		$this->load->view('modals/enviar_missatge', $data);
	}
	
	/* 
	 * Recepcio per post del asunte i missatge, 
	 * els guardem a la base de dades si no hi ha errors i retornem un ok de missatge enviat
	 */
	public function enviar_missatge_submit(){
		$iddestinatari = filter_var($this->input->post('destinatari'), FILTER_SANITIZE_NUMBER_INT);	
		$asunte = filter_var($this->input->post('asunte'), FILTER_SANITIZE_STRING);
		$missatge = filter_var($this->input->post('missatge'), FILTER_SANITIZE_STRING);	
		if(empty($iddestinatari) || empty($asunte) || empty($missatge)){
			$this->load->view('modals/enviar_missatge_error');
			return false;
		}
		
		//Si el destinatari no esxisteix error!
		$this->load->model('Usuari_model','',TRUE);
		$destinatariInfo = $this->Usuari_model->dadesUsuari($iddestinatari);
		if($destinatariInfo == null){
			$this->load->view('modals/enviar_missatge_error');
			return false;
		}	
		
		//error al guardar a la base de dades
		$this->load->model('Missatge_model','',TRUE);
		if(!$this->Missatge_model->crearMissatge($this->session->userdata('idusuario'), $iddestinatari, $asunte, $missatge)){
			$this->load->view('modals/enviar_missatge_error');
			return false;
		}	
		
		/*missatge guardat mostrem modal OK*/
		$this->load->view('modals/enviar_missatge_ok');
	}
	
	/* 
	 * Carrega les dades d'un missatge, i les retorna en una vista codificada en JSON
	 */
	public function llegir_missatge($idmissatge){
		$idmissatge = filter_var($idmissatge, FILTER_SANITIZE_NUMBER_INT);		
		if(empty($idmissatge)){
			$this->load->view('modals/enviar_missatge_error');
			return false;
		}
		
		//si no existeix el missatge a la base de dades error
		$this->load->model('Missatge_model','',TRUE);
		if(($missatge = $this->Missatge_model->infoMissatge($idmissatge))==null){
			$this->load->view('modals/enviar_missatge_error');
			return false;
		}
		
		//Si l'usuari que intenta carregar el missatge no es ni el remitent ni el destinatari no es deixa veure i envia error
		if($missatge['iddestinatari']!=$this->session->userdata('idusuario') && $missatge['idremitent']!=$this->session->userdata('idusuario')){
			$this->load->view('modals/enviar_missatge_error');
			return false;
		}	
		$data['missatge'] = $missatge;
		
		//Sino ha estat llegit abans el marquem com a llegit
		if($missatge['llegit']=="0" && $missatge['iddestinatari']==$this->session->userdata('idusuario')){
			$this->Missatge_model->marcarLlegit($idmissatge);
		}
		
		$this->load->view('modals/llegir_missatge', $data);
	}
	
	/* 
	 * Elimina un missatge segons si es el remitent o el destinatari. 
	 */
	public function missatge_delete(){
		$idmissatge = filter_var($this->input->post('idmissatge'), FILTER_SANITIZE_NUMBER_INT);		
		if(empty($idmissatge)){
			$this->_print_error("Error, missatge no valid");
		}
		
		$this->load->model('Missatge_model','',TRUE);
		if(($missatge = $this->Missatge_model->infoMissatge($idmissatge))==null){
			$this->_print_error("Error, missatge no valid");
		}
		
		if($missatge['iddestinatari']==$this->session->userdata('idusuario')){
			$this->Missatge_model->borrarRebut($idmissatge);						
			$json = array('error'=>'0');
        	echo json_encode($json);
        	return true;	
        	
		}elseif($missatge['idremitent']==$this->session->userdata('idusuario')){
			$this->Missatge_model->borrarEnviat($idmissatge);
			$json = array('error'=>'0');
       		echo json_encode($json);
       		return true;	
       		
		}else{
			$this->_print_error("Error, missatge no valid");
		}
	}
	
	/* 
	 * Retorna errors en format json per ser interpretats per el js de les peticions ajax 
	 */
	private function _print_error($msg=""){
		$tmpl_msg = $this->load->view('modals/errors', array("error_msg"=>$msg), TRUE);
        $json = array('error'=>'1', 'msg'=>$tmpl_msg);
        echo json_encode($json);
        exit;
    }
			
}