<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Professor extends CI_Controller {
	private $tipo = 2;
	
	public function __construct(){
		parent::__construct();			
		/* Comprovació d'acces */
		if(!is_logged() || $this->session->userdata('tipus')!=$this->tipo){
			redirect(base_url());
			exit;
		}		
	}
	
	/* professors dashboard */
	public function index()
	{
		$data['page']="professor/dashboard";
		$this->load->view('home', $data);		
	}
	
	/*
	 * Llistat d'examens actius
	 */
	public function examens_actius()
	{
		$this->load->model('Examen_model','',TRUE);
		$llista['examens'] = $this->Examen_model->llistarExamensActius($this->session->userdata('idusuario'));
		$data['examensactius'] = $this->load->view('professor/examens_actius_llista', $llista, TRUE);
		$data['page']="professor/examens_actius";
		$this->load->view('home', $data);		
	}
	
	/*
	 * Eliminar un examen i retorna llista d'examens actius en html-json
	 */
	public function examen_delete()
	{
		$this->load->model('Examen_model','',TRUE);
				
		$idexamen = filter_var($this->input->post('idexamen'), FILTER_SANITIZE_NUMBER_INT);		
		if(empty($idexamen)){				
			$this->_print_error("Error al eliminar l'examen");
		}
		$this->Examen_model->deleteExamen($idexamen);
				
		$llista['examens'] = $this->Examen_model->llistarExamensActius($this->session->userdata('idusuario'));
		$examensactius = $this->load->view('professor/examens_actius_llista', $llista, TRUE);	
		$json = array('error'=>'0', 'examens'=>$examensactius);
		echo json_encode($json); 	
	}
	
	/* 
	 * Llistat d'examens acabats
	 */
	public function examens_acabats()
	{
		$this->load->model('Examen_model','',TRUE);
		$data['examens'] = $this->Examen_model->llistarExamensAcabats($this->session->userdata('idusuario'));
		$data['page']="professor/examens_acabats";
		$this->load->view('home', $data);		
	}
	
	/* 
	 * Nomes carreca la vista de creacio d'examens amb els formularis 
	 */
	public function crear_examen()
	{
		$this->load->model('Assignatura_model','',TRUE);
		$data['assignatres'] = $this->Assignatura_model->assignaturesUsuari($this->session->userdata('idusuario'));
		$data['page']="professor/crear_examen";
		$this->load->view('home', $data);		
	}
	
	/*
	 * Rep per ajax les dades basiques de creacio d'un examen i les guarda.
	 * Retorna per json el id del examen creat per afegir les futures preguntes i les dades en text
	 */
	public function guardar_examen()
	{		
		$this->load->model('Examen_model','',TRUE);
		$sqldata['titul'] = filter_var($this->input->post('titul'), FILTER_SANITIZE_STRING);
		$sqldata['idassignatura'] = filter_var($this->input->post('assignatura'), FILTER_SANITIZE_NUMBER_INT);
		$sqldata['data_inici'] = format_data_usa(filter_var($this->input->post('datainici'), FILTER_SANITIZE_STRING));
		$sqldata['data_fi'] = format_data_usa(filter_var($this->input->post('datafi'), FILTER_SANITIZE_STRING));
		$action = $this->input->post('action');
		
		/* fallo de recepcio de dades*/	
		$errmsg = "";	
		foreach($sqldata as $key => $val){
			if(empty($val)){
				$errmsg .= "Error al rebre el camp $key \n\r";
			}
		}		
		if($errmsg != ""){
			$this->_print_error($errmsg);
		}
		/* error data inici anterior a actual */		
		if(strtotime($sqldata['data_inici'])<time()){
			$this->_print_error("La data d'inici a de ser posterior a la actual");
		}	
		/* error data fi anterior a data inici*/		
		if(strtotime($sqldata['data_inici'])>strtotime($sqldata['data_fi'])){
			$this->_print_error("La data de fi ha de ser posterior a la d'inici");
		}				
		
		if($action == "add"){
			$sqldata['idprofessor'] = $this->session->userdata('idusuario');
			/* Si falla al introduir */			
			if(!$idexamen = $this->Examen_model->crearExamen($sqldata)){
				$this->_print_error("Error al insertar l'Examen");
			}		
		}elseif($action == "edit"){
			$idexamen = filter_var($this->input->post('idexamen'), FILTER_SANITIZE_NUMBER_INT);		
			if(empty($idexamen)){			
				$this->_print_error("Error al editar examen");
			}
			
			/* Si falla al modificar */
			if(!$this->Examen_model->editarExamen($idexamen, $sqldata)){
				$this->_print_error("Error al editar examen");
			}	
		}else{
			$this->_print_error("Error inesperat");
		}
		
		/* Si arribem aqui sense errors*/
		$this->load->model('Assignatura_model','',TRUE);		
		$exdata = $sqldata;
		$assignatura = $this->Assignatura_model->dadesAssignatura($sqldata['idassignatura']);
		$exdata['assignatura'] = $assignatura['nom'];
		$examendata = $this->load->view('professor/dades_examen', $exdata, TRUE);	
		$json = array('error'=>'0', 'idexamen'=>$idexamen, 'examendata'=>$examendata);
		echo json_encode($json);	
	}
	
	/*
	 * Funcio que finalitza creacio d'examen, envia mails als alumnes de la assignatura i retorna una modal de error o ok
	 */
	public function finalitzar_examen($idexamen = null){
		if($idexamen == null){
			$this->_print_error("Error al rebre id de examen");
		}
		$idexamen = filter_var($idexamen, FILTER_SANITIZE_NUMBER_INT);
		$this->load->model('Examen_model','',TRUE);
		$examen = $this->Examen_model->infoExamen($idexamen);
		
		/* error examen no existeix*/
		if($examen == null){
			$this->_print_error("Error examen no valid", "L'examen no existeix");
		} 
		
		//carregar llistat de usuaris de la assignatura
		$this->load->model('Usuari_model','',TRUE);
		$usuaris = $this->Usuari_model->alumnesAssignatura($examen['idassignatura']);
		
		//Bucle d'enviament de mails als usuaris per avisar
		foreach($usuaris as $u){
			$mail_msg = "Hola ".$u['nom'].", \n " .
					"El professor ".$this->session->userdata('nom')." ha crear un nou examen de la assignatura ".$examen['nomassignatura']." \n\n " .
					"La data d'inici es el dia: ".format_data_europeu($examen['data_inici'])." \n" .
					"Finalitza el dia: ".format_data_europeu($examen['data_fi'])." \n\n " .
					"Et recomenem que no esperis al ultim día per fer-lo. \n " .
					"Molta sort! ";
		    $this->load->library('email');		
		    $this->email->from('no-reply@tfc.enredats.com', 'GenQuiz');
		    $this->email->to($u['mail']);
		    $this->email->subject("Tens un nou examen");
		    $this->email->message($mail_msg);
		    $this->email->send();
		}
		
		$this->load->view('modals/examen_finalitzat');
		
	}
	
	/*
	 * Carrega les dades del examen junt amb les seves preguntes per que la vista les mostri i es puguin editar
	 * Realitza comprovacions
	 */
	public function editar_examen($idexamen = null)
	{
		/* error examen null*/
		if($idexamen == null){
			$this->_show_html_error("Error examen no valid", "No s'ha rebut el numero d'examen a editar");
			return false;
		}
		$this->load->model('Examen_model','',TRUE);
		$data['examen'] = $this->Examen_model->infoExamen($idexamen);
		
		/* error examen no existeix*/
		if($data['examen'] == null){
			$this->_show_html_error("Error examen no valid", "L'examen no existeix");
			return false;
		} 
		
		/* error examen de un altre professor, no tens permis per editarlo*/
		if($data['examen']['idprofessor']!=$this->session->userdata('idusuario')){
			$this->_show_html_error("Error examen no valid", "No ets el creador del examen, no tens permis per editarlo");
			return false;
		}
		
		/* error examen ja començat, no es pot editar*/
		if(strtotime($data['examen']['data_inici'])<time()){
			$this->_show_html_error("Error examen ja començat", "El examen que intentes editar ja ha començat, per seguretat no es permet editar-lo");
			return false;
		}
		
		/* si no hi ha errors*/
		$this->load->model('Assignatura_model','',TRUE);
		$data['assignatres'] = $this->Assignatura_model->assignaturesUsuari($this->session->userdata('idusuario'));
		
		$data['preguntes'] = $this->Examen_model->llistatPreguntes($idexamen);
		foreach($data['preguntes'] as $key => $p){
			$data['preguntes'][$key]['respostes'] = $this->Examen_model->llistatRespostes($p['idpregunta']);
		}
		
		$data['page']="professor/editar_examen";
		$this->load->view('home', $data);		
	}
	
	/*
	 * Metode ajax que afegeix una pregunta i les seves respostes junt amb la corecte
	 * Rep per post els parametres i retorna un json amb error o no.
	 */
	public function afegir_pregunta()
	{
		$this->load->model('Examen_model','',TRUE);
		$sqldata['pregunta'] = filter_var($this->input->post('pregunta'), FILTER_SANITIZE_STRING);
		$sqldata['resposta'] = $this->input->post('resposta');
		$sqldata['correcte'] = filter_var($this->input->post('correcte'), FILTER_SANITIZE_NUMBER_INT);
		$sqldata['idexamen'] = filter_var($this->input->post('p_idexamen'), FILTER_SANITIZE_NUMBER_INT);
		
		/* fallo de recepcio de dades*/	
		$errmsg = "";	
		foreach($sqldata as $key => $val){
			if(empty($val)){
				$errmsg .= "Error al rebre el camp $key \n\r";
			}
		}		
		if($errmsg != ""){
			$this->_print_error($errmsg);
		}
		//insertar pregunta
		$sqlpregunta = array('idexamen'=>$sqldata['idexamen'], 'pregunta'=>$sqldata['pregunta']);
		$idpregunta = $this->Examen_model->crearPregunta($sqlpregunta);
		$pregunta = array('idpregunta'=>$idpregunta, 'text'=>$sqldata['pregunta']);
		//insertar respostes
		foreach($sqldata['resposta'] as $key => $text){
			if($sqldata['correcte'] == $key){
				$r_correcte = "1";
			}else{
				$r_correcte = "0";
			}	
			$respostatext = filter_var($text, FILTER_SANITIZE_STRING);
			$sqlresposta = array('idexamen'=>$sqldata['idexamen'], 'idpregunta'=>$idpregunta, 'resposta'=>$respostatext, 'correcte'=>$r_correcte);
			$idresposta = $this->Examen_model->crearResposta($sqlresposta);
			$respostes[] = array('idexamen'=>$sqldata['idexamen'], 'idresposta'=>$idresposta, 'text'=>$respostatext, 'correcte'=>$r_correcte);
		}
		
		//Preparar retorn:
		$preg_data = array('idexamen'=>$sqldata['idexamen'], 'pregunta'=>$pregunta, 'respostes'=>$respostes);
		$preguntadata = $this->load->view('professor/pregunta_template', $preg_data, TRUE);	
		$json = array('error'=>'0', 'pregunta'=>$preguntadata);
		echo json_encode($json);			
	}
	
	/*
	 * Metode per AJAX que edita les dades d'una pregunta, si tot va be respon json amb noves dades sino json error
	 */
	public function editar_pregunta($idpregunta = null){
		$this->load->model('Examen_model','',TRUE);
		if($idpregunta == null) $this->_print_error("Error editan pregunta");
		$sqldata['pregunta'] = filter_var($this->input->post('pregunta'), FILTER_SANITIZE_STRING);
		$sqldata['resposta'] = $this->input->post('resposta');
		$sqldata['correcte'] = filter_var($this->input->post('correcte'), FILTER_SANITIZE_NUMBER_INT);
		
		/* fallo de recepcio de dades*/	
		$errmsg = "";	
		foreach($sqldata as $key => $val){
			if(empty($val)){
				$errmsg .= "Error al rebre el camp $key \n\r";
			}
		}		
		if($errmsg != ""){
			$this->_print_error($errmsg);
		}
		
		//editar pregunta
		$sqlpregunta = array('pregunta'=>$sqldata['pregunta']);
		$this->Examen_model->editarPregunta($idpregunta, $sqlpregunta);
		$pregunta = array('idpregunta'=>$idpregunta, 'text'=>$sqldata['pregunta']);
		//editar respostes
		foreach($sqldata['resposta'] as $key => $text){
			$idresposta = $key;
			if($sqldata['correcte'] == $idresposta){
				$r_correcte = "1";
			}else{
				$r_correcte = "0";
			}	
			$respostatext = filter_var($text, FILTER_SANITIZE_STRING);
			$sqlresposta = array('resposta'=>$respostatext, 'correcte'=>$r_correcte);
			$this->Examen_model->editarResposta($idresposta, $sqlresposta);
			$respostes[] = array('idresposta'=>$idresposta, 'text'=>$respostatext, 'correcte'=>$r_correcte);
		}
		
		//Preparar retorn:
		$preg_data = array('pregunta'=>$pregunta, 'respostes'=>$respostes);
		$preguntadata = $this->load->view('professor/pregunta_data_template', $preg_data, TRUE);	
		$json = array('error'=>'0', 'pregunta'=>$preguntadata);
		echo json_encode($json);
	}
	
	/* 
	 * Metode AJAX que elimina una pregunta de la base de dades
	 */
	public function pregunta_delete()
	{
		$this->load->model('Examen_model','',TRUE);
				
		$idpregunta = filter_var($this->input->post('idpregunta'), FILTER_SANITIZE_NUMBER_INT);		
		if(empty($idpregunta)){				
			$this->_print_error("Error al eliminar la pregunta");
		}
		if(!$this->Examen_model->deletePregunta($idpregunta)){
			$this->_print_error("Error al eliminar la pregunta");
		}
		$json = array('error'=>'0');
		echo json_encode($json); 		
	}
	
	public function publicar_notes(){
		$this->load->model('Examen_model','',TRUE);
		$this->load->model('Usuari_model','',TRUE);
		$idexamen = filter_var($this->input->post('idexamen'), FILTER_SANITIZE_NUMBER_INT);
		if($idexamen==null || ($infoexamen = $this->Examen_model->infoExamenNotes($idexamen,$this->session->userdata('idusuario')))==null){
			$this->_print_error("Examen no valid, no ha acabat el termini de realització o no tens acces a aquest examen");
		}
		if($infoexamen['tancat']=="1"){
			$this->_print_error("Ja han estat publicades les notes d'aquest examen anteriorment");
		}
		
		$arr_alumnes = $this->Usuari_model->alumnesAssignatura($infoexamen['idassignatura']);
		$arr_respostes = $this->Examen_model->respostesCorrectesExamen($idexamen);
		
		$this->load->helper('correccio');
		$arr_respostes = adaptar_array_respostes($arr_respostes);
		
		foreach($arr_alumnes as $alumne){
			$arr_alumne_respostes = $this->Examen_model->respostesAlumneExamen($idexamen, $alumne['idusuari']);			
			$arr_r_a = adaptar_array_respostes($arr_alumne_respostes);
			$sqlresultats[$alumne['idusuari']] = comparar_respostes($arr_respostes, $arr_r_a);				
		}
		
		if(!$this->Examen_model->publicarNotes($idexamen,$sqlresultats)){
			$this->_print_error("Error al corretgir el examen");
		}
		/*si arribem aquí vol dir que tot ha funcionat correctament*/
		$ok_modal = $this->load->view('modals/notas_publicades',array("idexamen"=>$idexamen) ,TRUE);
		$json = array('error'=>'0', 'msg'=>$ok_modal);
        echo json_encode($json);		
	}
	
	/*
	 * Pagina de resultat d'examen. Mostra les notes que han tret els alumnes
	 */
	public function resultats_examen($idexamen = null){
		$this->load->model('Examen_model','',TRUE);
		$this->load->model('Usuari_model','',TRUE);
		$idexamen = filter_var($idexamen, FILTER_SANITIZE_NUMBER_INT);
		
		if($idexamen==null || ($infoexamen = $this->Examen_model->infoExamenNotes($idexamen,$this->session->userdata('idusuario')))==null){
			$this->_show_html_error("Error examen no valid", "No ha acabat el termini de realització o no tens acces a aquest examen");
			return false;
		}
		
		if($infoexamen['tancat']=="0"){			
			$this->_show_html_error("Error examen no corregit encara");
			return false;
		}
		
		$data['notesexamen'] = $this->Examen_model->usuarisNotesExamen($idexamen);
		
		$data['examen'] = $infoexamen;
		$data['page']="professor/resultat_examen";
		$this->load->view('home', $data);
	}
	
	/*
	 * Mostre la correccio del examen presentat per un alumne.
	 * Carrega les respostes oficials i les del alumne i la vista o mostre formatejant en colors els erros i acerts
	 */
	public function respostes_alumne($idexamen = null, $idalumne = null){
		$this->load->helper('correccio');
		$this->load->model('Examen_model','',TRUE);
		$this->load->model('Usuari_model','',TRUE);
		$idexamen = filter_var($idexamen, FILTER_SANITIZE_NUMBER_INT);
		$idalumne = filter_var($idalumne, FILTER_SANITIZE_NUMBER_INT);
		
		if($idexamen==null || ($infoexamen = $this->Examen_model->infoExamenNotes($idexamen,$this->session->userdata('idusuario')))==null){
			$this->_show_html_error("Error examen no valid", "No ha acabat el termini de realització o no tens acces a aquest examen");
			return false;
		}
		$data['examen']=$infoexamen;
		
		if(($infoalumne = $this->Examen_model->alumneNotaExamen($idexamen,$idalumne))==null){
			$this->_show_html_error("Error alumne no valid", "El alumne no es valid");
			return false;
		}
		$data['alumne'] = $infoalumne;
		
		$data['preguntes'] = $this->Examen_model->llistatPreguntes($idexamen);
		foreach($data['preguntes'] as $key => $p){
			$data['preguntes'][$key]['respostes'] = $this->Examen_model->llistatRespostes($p['idpregunta']);
		}	
		
		$alumne_respostes = $this->Examen_model->respostesAlumneExamen($idexamen, $idalumne);
		$data['a_resp'] = adaptar_array_respostes($alumne_respostes);
	
		$data['page']="professor/respostes_alumne";
		$this->load->view('home', $data);
	}
	
	
	/* 
	 * Retorna errors en format html codificat en json per als colorbox. 
	 */
	private function _show_html_error($titul="", $msg=""){
		$this->load->view('home', array("page"=>"professor/error","titul"=>$titul,"msg"=>$msg));
    }
	
	/* 
	 * Retorna errors en format json per ser interpretats per el js de les peticions ajax 
	 */
	private function _print_error($msg=""){
		$tmpl_msg = $this->load->view('modals/errors', array("error_msg"=>$msg), TRUE);
        $json = array('error'=>'1', 'msg'=>$tmpl_msg);
        echo json_encode($json);
        exit;
    }	
}

?>